@@ -362,7 +362,7 @@ static inline unsigned int swrate_add_scaled(unsigned int *sum, unsigned int n,
 
 	old_sum = *sum;
 	do {
-		new_sum = old_sum + v * s - div64_32((unsigned long long)(old_sum + n) * s, n);
+		new_sum = old_sum + v * s - div64_32((unsigned long long)old_sum * s + n - 1, n);
 	} while (!HA_ATOMIC_CAS(sum, &old_sum, new_sum) && __ha_cpu_relax());
 	return new_sum;
 }
