@@ -239,7 +239,7 @@ static inline int quic_read_uint32(uint32_t *val,
 	if (end - *buf < sizeof *val)
 		return 0;
 
-	*val = ntohl(*(uint32_t *)*buf);
+	*val = ntohl(read_u32(*buf));
 	*buf += sizeof *val;
 
 	return 1;
@@ -256,7 +256,7 @@ static inline int quic_write_uint32(unsigned char **buf,
 	if (end - *buf < sizeof val)
 		return 0;
 
-	*(uint32_t *)*buf = htonl(val);
+	write_u32(*buf, htonl(val));
 	*buf += sizeof val;
 
 	return 1;
@@ -318,30 +318,12 @@ static inline size_t max_available_room(size_t sz, size_t *len_sz)
  */
 static inline size_t max_stream_data_size(size_t sz, size_t ilen, size_t dlen)
 {
-	size_t ret, len_sz, dlen_sz;
+	size_t ret, dlen_sz;
 
-	/*
-	 * The length of variable-length QUIC integers are powers of two.
-	 * Look for the first 3length" field value <len_sz> which match our need.
-	 * As we must put <ilen> bytes in our buffer, the minimum value for
-	 * <len_sz> is the number of bytes required to encode <ilen>.
-	 */
-	for (len_sz = quic_int_getsize(ilen);
-	     len_sz <= QUIC_VARINT_MAX_SIZE;
-	     len_sz <<= 1) {
-		if (sz < len_sz + ilen)
-			return 0;
-
-		ret = max_available_room(sz - len_sz - ilen, &dlen_sz);
-		if (!ret)
-			return 0;
-
-		/* Check that <*len_sz> matches <ret> value */
-		if (len_sz + ilen + dlen_sz + ret <= quic_max_int(len_sz))
-			return ret < dlen ? ret : dlen;
-	}
-
-	return 0;
+	ret = max_available_room(sz - ilen, &dlen_sz);
+	if (!ret)
+		return 0;
+	return ret < dlen ? ret : dlen;
 }
 
 /* Return the length in bytes of <pn> packet number depending on
@@ -432,6 +414,7 @@ static inline void quic_tx_packet_dgram_detach(struct quic_tx_packet *pkt)
 		pkt->prev->next = pkt->next;
 	if (pkt->next)
 		pkt->next->prev = pkt->prev;
+	pkt->prev = pkt->next = NULL;
 }
 
 
@@ -444,6 +427,7 @@ static inline void quic_tx_packet_refinc(struct quic_tx_packet *pkt)
 /* Decrement the reference counter of <pkt> */
 static inline void quic_tx_packet_refdec(struct quic_tx_packet *pkt)
 {
+	BUG_ON(pkt->refcnt <= 0);
 	if (!HA_ATOMIC_SUB_FETCH(&pkt->refcnt, 1)) {
 		BUG_ON(!LIST_ISEMPTY(&pkt->frms));
 		/* If there are others packet in the same datagram <pkt> is attached to,
@@ -466,6 +450,7 @@ static inline void quic_path_init(struct quic_path *path, int ipv4,
 	quic_loss_init(&path->loss);
 	path->mtu = max_dgram_sz;
 	path->cwnd = QUIC_MIN(10 * max_dgram_sz, QUIC_MAX(max_dgram_sz << 1, 14720U));
+	path->mcwnd = path->cwnd;
 	path->min_cwnd = max_dgram_sz << 1;
 	path->prep_in_flight = 0;
 	path->in_flight = 0;
@@ -636,6 +621,8 @@ void qc_release_frm(struct quic_conn *qc, struct quic_frame *frm);
 
 void qc_check_close_on_released_mux(struct quic_conn *qc);
 
+void qc_release_pktns_frms(struct quic_conn *qc, struct quic_pktns *pktns);
+
 void quic_conn_release(struct quic_conn *qc);
 
 void qc_kill_conn(struct quic_conn *qc);
