@@ -623,6 +623,8 @@ static int sink_forward_session_init(struct appctx *appctx)
 
 	if (!sockaddr_alloc(&addr, &sft->srv->addr, sizeof(sft->srv->addr)))
 		goto out_error;
+	/* srv port should be learned from srv->svc_port not from srv->addr */
+	set_host_port(addr, sft->srv->svc_port);
 
 	if (appctx_finalize_startup(appctx, sft->sink->forward_px, &BUF_NULL) == -1)
 		goto out_free_addr;
@@ -702,6 +704,7 @@ static struct appctx *sink_forward_session_create(struct sink *sink, struct sink
 	if (appctx_init(appctx) == -1)
 		goto out_free_appctx;
 
+	sft->last_conn = now_ms;
 	return appctx;
 
 	/* Error unrolling */
@@ -724,9 +727,21 @@ static struct task *process_sink_forward(struct task * task, void *context, unsi
 	if (!stopping) {
 		while (sft) {
 			HA_SPIN_LOCK(SFT_LOCK, &sft->lock);
-			/* if appctx is NULL, start a new session */
-			if (!sft->appctx)
-				sft->appctx = sink_forward_session_create(sink, sft);
+			/* if appctx is NULL, start a new session
+			 *
+			 * We enforce a tempo to ensure we don't perform more than 1 session
+			 * establishment attempt per second.
+			 */
+			if (!sft->appctx) {
+				uint tempo = sft->last_conn + MS_TO_TICKS(1000);
+
+				if (sft->last_conn == TICK_ETERNITY || tick_is_expired(tempo, now_ms))
+					sft->appctx = sink_forward_session_create(sink, sft);
+				else if (task->expire == TICK_ETERNITY)
+					task->expire = tempo;
+				else
+					task->expire = tick_first(task->expire, tempo);
+			}
 			HA_SPIN_UNLOCK(SFT_LOCK, &sft->lock);
 			sft = sft->next;
 		}
